@@ -32,6 +32,7 @@
 #include <arpa/inet.h>
 
 #include "private.h"
+#include "useful.h"
 #include "random.h"
 #include "time.h"
 #include "time-private.h"
@@ -360,6 +361,8 @@ static int dhcp_client_send_discover(struct l_dhcp_client *client)
 						client->hostname))
 			return -EINVAL;
 
+	_dhcp_message_builder_append(&builder, DHCP_OPTION_RAPID_COMMIT,
+					0, "");
 	_dhcp_message_builder_finalize(&builder, &len);
 
 	return client->transport->l2_send(client->transport,
@@ -740,8 +743,10 @@ static int dhcp_client_receive_ack(struct l_dhcp_client *client,
 						client, NULL);
 		if (client->rtnl_add_cmdid)
 			client->rtnl_configured_address = a;
-		else
+		else {
 			CLIENT_DEBUG("Configuring address via RTNL failed");
+			l_rtnl_address_free(a);
+		}
 	}
 
 	return r;
@@ -770,15 +775,17 @@ static int dhcp_client_receive_offer(struct l_dhcp_client *client,
 	return 0;
 }
 
-static void dhcp_client_rx_message(const void *data, size_t len, void *userdata)
+static void dhcp_client_rx_message(const void *data, size_t len, void *userdata,
+					const uint8_t *saddr)
 {
 	struct l_dhcp_client *client = userdata;
 	const struct dhcp_message *message = data;
 	struct dhcp_message_iter iter;
+	char buf[INET_ADDRSTRLEN];
 	uint8_t msg_type = 0;
 	uint8_t t, l;
 	const void *v;
-	int r;
+	int r, e;
 	struct in_addr ia;
 
 	CLIENT_DEBUG("");
@@ -811,6 +818,17 @@ static void dhcp_client_rx_message(const void *data, size_t len, void *userdata)
 	case DHCP_STATE_INIT:
 		return;
 	case DHCP_STATE_SELECTING:
+		if (msg_type == DHCP_MESSAGE_TYPE_ACK) {
+			_dhcp_message_iter_init(&iter, message, len);
+
+			while (_dhcp_message_iter_next(&iter, &t, &l, &v))
+				if (t == DHCP_OPTION_RAPID_COMMIT) {
+					CLIENT_ENTER_STATE(
+							DHCP_STATE_REQUESTING);
+					goto receive_rapid_commit;
+				}
+		}
+
 		if (msg_type != DHCP_MESSAGE_TYPE_OFFER)
 			return;
 
@@ -831,6 +849,7 @@ static void dhcp_client_rx_message(const void *data, size_t len, void *userdata)
 	case DHCP_STATE_REQUESTING:
 	case DHCP_STATE_RENEWING:
 	case DHCP_STATE_REBINDING:
+	receive_rapid_commit:
 		if (msg_type == DHCP_MESSAGE_TYPE_NAK) {
 			CLIENT_DEBUG("Received NAK, Stopping...");
 			l_dhcp_client_stop(client);
@@ -851,12 +870,19 @@ static void dhcp_client_rx_message(const void *data, size_t len, void *userdata)
 		l_timeout_remove(client->timeout_resend);
 		client->timeout_resend = NULL;
 
-		if (client->transport->bind)
-			client->transport->bind(client->transport,
+		if (client->transport->bind) {
+			e = client->transport->bind(client->transport,
 						client->lease->address);
+			if (e < 0) {
+				CLIENT_DEBUG("Failed to bind dhcp socket. "
+					"Error %d: %s", e, strerror(-e));
+			}
+		}
 
 		dhcp_client_event_notify(client, r);
 
+		client->lease->bound_time = l_time_now();
+
 		/*
 		 * Start T1, once it expires we will start the T2 timer.  If
 		 * we renew the lease, we will end up back here.
@@ -906,11 +932,12 @@ static void dhcp_client_rx_message(const void *data, size_t len, void *userdata)
 		l_acd_set_skip_probes(client->acd, true);
 
 		ia.s_addr = client->lease->address;
+		inet_ntop(AF_INET, &ia, buf, INET_ADDRSTRLEN);
 
 		/* For unit testing we don't want this to be a fatal error */
-		if (!l_acd_start(client->acd, inet_ntoa(ia))) {
+		if (!l_acd_start(client->acd, buf)) {
 			CLIENT_DEBUG("Failed to start ACD on %s, continuing",
-						inet_ntoa(ia));
+						buf);
 			l_acd_destroy(client->acd);
 			client->acd = NULL;
 		}
